作为初学者一直,经过多次的上网搜索你一定会看到迈克老狼的向量加法的示例,不知道你是否和我一样,刚开始并不是很准确的知道他的add.cl写的代码的意思,源码如下:

#pragma OPENCL EXTENSION cl_amd_printf : enable
__kernel void vecadd(__global const float* a, __global const float* b, __global float* c) { int x = get_global_id(); int y = get_global_id(); int width = get_global_size(); int height = get_global_size(); if(x == && y ==) printf("%d, %d,%d,%d,%d,%d\n",get_local_size(),get_local_size(),get_local_id(), get_local_id(),get_group_id(),get_group_id()); c[x + y * width] = a[x + y * width] + b[x + y * width]; } 我们在k

这里面我把二维的这些值全部罗列出来如下:

int idx = get_global_id(0);
int idy = get_global_id(1);

uint wiWidth = get_global_size(0);
uint wiHeight = get_global_size(1);

uint gix_t = get_group_id(0);
uint giy_t = get_group_id(1);

uint num_of_blocks_x = get_num_groups(0);
uint num_of_blocks_y = get_num_groups(1);

uint lix = get_local_id(0);
uint liy = get_local_id(1);

你会奇怪为什么他的add.cl优化会这么写:

c[x + y * width] = a[x + y * width] + b[x + y * width];

下面进行分析:

add.cpp关键的代码部分如下(只罗列出部分代码):

#define width 8//256
#define height 16//256
#define NWITEMS width*height//262144 size_t globalThreads[] = {width, height};
size_t localx, localy; localx = 2;
localy = 4; size_t localThreads[] = {localx, localy}; clEnqueueNDRangeKernel( queue,
kernel,
2,
NULL,
globalThreads,
localThreads, 0, NULL, &ev);

我自己的add.cl代码如下(里面添加啦打印,以便我们可以看出结果):

   int idx = get_global_id();
int idy = get_global_id(); uint wiWidth = get_global_size();
uint wiHeight = get_global_size();
printf("Magnum Global idx = %d, idy = %d, sizeX =%d,sizeY =%d\n",idx,idy,wiWidth,wiHeight); uint gix_t = get_group_id();
uint giy_t = get_group_id();
uint num_of_blocks_x = get_num_groups();
uint num_of_blocks_y = get_num_groups(); printf("Magnum Group idx = %d, idy = %d, blockX=%d,blockY=%d\n",gix_t,gix_t,num_of_blocks_x,num_of_blocks_y); uint lix = get_local_id();
uint liy = get_local_id(); uint LocalX = get_local_size();
uint LocalY = get_local_size(); printf("Magnum Local idx = %d, idy = %d, localX=%d,localY=%d\n\n",lix,liy,LocalX,LocalY);

输出的结果如下:

Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY= Magnum Global idx = , idy = , sizeX =,sizeY =
Magnum Group idx = , idy = , blockX=,blockY=
Magnum Local idx = , idy = , localX=,localY=

从从这个结果首先能说明几个问题:

首先我们在:

  1. clEnqueueNDRangeKernel( queue, kernel, , NULL, globalThreads, localThreads, 0, NULL, &ev); 设置维度是2,否则add.cl中所有类似get_local_size(1)将只会返回一。
  2. 设置GlobaThreads 和 LocalThreads 不一定是 GlobalThreads= {8, 8}; 也可以不相等,LocalThreads也是一样。
  3. 理解opencl中的内存分布可以理解为二维数组,而且内存寻址的方向就像是:左上角(0,0)为原点,一行一行的扫描下去。
  4. WorkGroup的大小,目前我认为X,Y可以分开来理解(后面若发现错误我会及时改正) WorkGroup number(x) = GlobalThread(x)/LocalThread(x).
  5. 最小的单位是LocalX * LocalY, 然后用总的GlobalThreads按照这个单位进行分割成一个一个的组。每个二维的小单位还是一行一行的扫描过去的。目前我们的这个是8*16,你会发现总的Thread也是8*16,对应的每一个线程执行一次加法操作,有人会问可不可以不用这么多呢,用4*16个Threads,每个线程里面做两次加法动作,其实是可以的,这个原则是根据你的具体的Device来说的,当前我们说的是Device是GPU,比如你的GPU当前支持最大的线程数是1024*1024,这种情况,你不用多的,而是用少的线程来做,会降低你的效率,你需要尽可能的让Device满负荷工作这样才能达到提高运算速度。

继续分析为什么 NDRange改成二维的之后,add.cl变成:

c[x + y * width] = a[x + y * width] + b[x + y * width];

因为在CPU端每个 C[8*16],通过打印可以在这里C[x+y*width],x 最大是7,width是8,y最大是17,只有这样才能做完所有的向量相加。

使用二维NDRange workgroup的更多相关文章

  1. OpenCL 学习step by step (5) 使用二维NDRange workgroup

    http://www.cnblogs.com/mikewolf2002/archive/2012/09/07/2675634.html 在本教程中,我们使用二维NDRange来设置workgroup, ...

  2. Javascript生成二维码(QR)

    网络上已经有非常多的二维码编码和解码工具和代码,很多都是服务器端的,也就是说需要一台服务器才能提供二维码的生成.本着对服务器性能的考虑,这种小事情都让服务器去做,感觉对不住服务器,尤其是对于大流量的网 ...

  3. iOS二维码生成、识别、扫描等

    二维码扫描 前言: 最近的项目中使用到了二维码,二维码这个模块功能也完成:觉得还是有必要总结一下用来做记录.好长时间没有写二维码了都忘记在差不多了,重新拾起来还是挻快的. 二维码使用场景: 生活中有很 ...

  4. 很多人很想知道怎么扫一扫二维码就能打开网站,就能添加联系人,就能链接wifi,今天说下这些格式,明天做个demo

    有些功能部分手机不能使用,网站,通讯录,wifi基本上每个手机都可以使用. 在看之前你可以扫一扫下面几个二维码先看看效果: 1.二维码生成 网址 (URL) 包含网址的 二维码生成 是大家平时最常接触 ...

  5. 免费开源的DotNet二维码操作组件ThoughtWorks.QRCode(.NET组件介绍之四)

    在生活中有一种东西几乎已经快要成为我们的另一个电子”身份证“,那就是二维码.无论是在软件开发的过程中,还是在普通用户的日常中,几乎都离不开二维码.二维码 (dimensional barcode) , ...

  6. 基于SignalR的消息推送与二维码描登录实现

    1 概要说明 使用微信扫描登录相信大家都不会陌生吧,二维码与手机结合产生了不同应用场景,基于二维码的应用更是比较广泛.为了满足ios.android客户端与web短信平台的结合,特开发了基于Singl ...

  7. 微信小程序的机会在于重新理解群组与二维码

    历时一年,唯一一个尚未发布就获得Pony Ma与Allen Zhang站台的产品:微信小程序,将于2017年1月9日正式上线了.我很期待.唯一要警惕的是:防止长考出臭棋. 在上线前夕,我对于如何借助小 ...

  8. javaScript生成二维码(支持中文,生成logo)

    资料搜索 选择star最多的两个 第一个就是用的比较多的jquery.qrcode.js(但不支持中文,不能带logo)啦,第二个支持ie6+,支持中文,根据第二个源代码,使得,jquery.qrco ...

  9. 微信小程序二维码推广统计

    微信小程序可以通过生成带参数的二维码,那么这个参数是可以通过APP的页面进行监控的 这样就可以统计每个二维码的推广效果. 今天由好推二维码推出的小程序统计工具HotApp小程序统计也推出了带参数二维码 ...

随机推荐

  1. JQuery 分页实现

     JQuery分页实现  css: .liList0 { padding-left:5px;}.liList0 li { width:160px; float:left; display:inline ...

  2. centos 交换分区

    内容来自:http://www.huzs.net/?p=1683 一般在桌面型的用不到,因为现在内存都比较大,服务器就不一样了,但是建议无论是在桌面还是服务器上,都设置 swap 以下操作都是在 ro ...

  3. 内容写到 csv 格式的文件中 及 读取 csv 格式的文件内容

    <?php/*把内容写到 csv 格式的文件中 基本思路是:1.用 $fp = fopen("filename", 'mode')打开一个csv文件,可以是打开时才建立的2. ...

  4. hbuilder工具快捷键 http://www.qq210.com/shoutu/android

    http://www.qq210.com/shoutu/android 创建HTML结构: h 8 (敲h激活代码块列表,按8选择第8个项目,即HTML代码块,或者敲h t Enter)中途换行: ' ...

  5. Spring Mvc Controller间跳转 重定向 传参 (转)

    原文链接:http://zghbwjl.blog.163.com/blog/static/12033667220137795252845/ 1. 需求背景     需求:spring MVC框架con ...

  6. mongodb3.2系统性学习——2、write concern mongodb 写安全机制

    为了尊重作者原文章位置:http://kyfxbl.iteye.com/blog/1952941 首先讲一下mongodb 的写操作过程: mongodb有一个write concern的设置,作用是 ...

  7. 定时生成bat命令

    windows下,定时生成bat的名. at 14:54 cmd /c "echo net share D=d:\ > d:d.bat" ^对>转义.

  8. 分享 - Social.framework

    /** *  第三方分享 * *  @param void 友盟分享 *  @param shareSDK *  @param 百度分享 */ #import "ViewController ...

  9. mcd, lm, VS lx

    LED常识之 mcd&lm&w的关系 转载自:http://1198.vip.blog.163.com/blog/static/202177117201211624535412/ LE ...

  10. BZOJ 1828: [Usaco2010 Mar]balloc 农场分配

    Description Input 第1行:两个用空格隔开的整数:N和M * 第2行到N+1行:第i+1行表示一个整数C_i * 第N+2到N+M+1行: 第i+N+1行表示2个整数 A_i和B_i ...